/*
** -- Local Includes --
*/

#include "qtchess_gui.h"

extern qtchess *chess;
extern qtchess_gui *gui;
extern qtchess_comm *comm;
extern QApplication *qapp;

int qtchess_gui::exec(void)
{
  return qapp->exec();
}

void qtchess_gui::init(void)
{
  QFrame *frame = NULL;
  QPopupMenu *filemenu = NULL, *helpmenu = NULL,
    *viewmenu = NULL, *setupmenu = NULL;
  QHBoxLayout *layout = NULL;

  /*
  ** Add a menubar.
  */

  if((filemenu = new QPopupMenu()) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((setupmenu = new QPopupMenu()) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((viewmenu = new QPopupMenu()) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  /*
  ** Add some menus.
  */

  filemenu->insertItem("&New Game", this, SLOT(newGame()));
  filemenu->insertSeparator();
  filemenu->insertItem("&Quit", this, SLOT(quit()));
  setupmenu->insertItem("&Connection Configuration", this, SLOT(setup()));
  viewmenu->insertItem("&Clocks", this, SLOT(showClocks()));
  viewmenu->insertItem("&History", this, SLOT(history()));

  if((helpmenu = new QPopupMenu()) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  helpmenu->insertItem("&About", this, SLOT(about()));
  helpmenu->insertItem("&Help", this, SLOT(help()));

  if((help_dialog = new qtchess_help_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((setup_dialog = new qtchess_setup_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((history_dialog = new qtchess_history_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((promote_dialog = new qtchess_promote_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((clocks_dialog = new qtchess_clocks_dialog(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  setup_dialog->setModal(true);
  this->menuBar()->insertItem("&File", filemenu);
  this->menuBar()->insertItem("&Setup", setupmenu);
  this->menuBar()->insertItem("&View", viewmenu);
  this->menuBar()->insertSeparator();
  this->menuBar()->insertItem("&Help", helpmenu);
  statusbar = this->statusBar();

  if(statusbar == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  statusbar->message("Status: Ready");
  qapp->setMainWidget(this);

  if((frame = new QFrame(this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((glboard = new openglWid(frame, "glboard")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  frame->setFrameStyle(QFrame::Sunken | QFrame::WinPanel);
  frame->setLineWidth(2);

  if((layout = new QHBoxLayout(frame, 2, 2, "layout")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  layout->addWidget(glboard, 1);

  /*
  ** Add the OpenGL board.
  */

  glboard->resize(OPEN_GL_DIMENSION, OPEN_GL_DIMENSION);
  this->setCaption("QtChess");
  this->setCentralWidget(frame);
  frame->setFixedSize(QSize(OPEN_GL_DIMENSION, OPEN_GL_DIMENSION));
  this->setFixedSize(QSize(OPEN_GL_DIMENSION, OPEN_GL_DIMENSION));
  this->show();
}

void qtchess_gui::display(void)
{
}

void qtchess_gui::history(void)
{
  history_dialog->hide();
  history_dialog->show();
}

void qtchess_gui::about(void)
{
  (void) QMessageBox::information(this,
				  "About",
				  "QtChess Version 2.05.\n"
				  "Copyright (c) 2003, 2004, 2006, 2007 "
				  "Alexis Megas.",
				  QMessageBox::Ok | QMessageBox::Default);
}

void qtchess_gui::showErrorMsg(const char *message)
{
  (void) QMessageBox::critical(this, "Error", message,
			       QMessageBox::Ok,
			       QMessageBox::Default);
}

void qtchess_gui::help(void)
{
  help_dialog->setup();
}

void qtchess_gui::quit(void)
{
  chess->quit(NULL, EXIT_SUCCESS);
}

void qtchess_gui::closeEvent(QCloseEvent *e)
{
  (void) e;
  chess->quit(NULL, EXIT_SUCCESS);
}

void qtchess_gui::newGame(void)
{
  int i = 0, j = 0;
  struct move_s current_move;

  if(chess->getFirst() == -1)
    {
      showErrorMsg("You must first start a game.");
      return;
    }
  else if(chess->getFirst() != I_AM_FIRST)
    {
      showErrorMsg("Only the first player may initiate a new game.");
      return;
    }

  chess->init();
  qapp->lock();
  gui->getGLBoard()->update();      
  qapp->unlock();

  for(i = 0; i < NSQUARES; i++)
    for(j = 0; j < NSQUARES; j++)
      current_move.board[i][j] = chess->board[i][j];

  current_move.x1 = current_move.x2 = current_move.y1 =
    current_move.y2 = current_move.r_x1 =
    current_move.r_x2 = current_move.r_y1 =
    current_move.r_y2 = current_move.piece =
    current_move.rook = -1;
  current_move.promoted = current_move.pawn_2 = 0;
  comm->sendMove(current_move);
  history_dialog->clear_method();
  clocks_dialog->init();

  /*
  ** Careful here. Set this after calling sendMove().
  */

  chess->setTurn(MY_TURN);
}

void qtchess_gui::setup(void)
{
  int h = 0, w = 0, x = 0, y = 0;

  h = gui->geometry().height();
  w = gui->geometry().width();
  x = gui->geometry().x();
  y = gui->geometry().y();
  setup_dialog->hide();
  setup_dialog->setGeometry(x + (w - setup_dialog->geometry().width()) / 2,
			    y + (h - setup_dialog->geometry().height()) / 2,
			    setup_dialog->geometry().width(),
			    setup_dialog->geometry().height());
  setup_dialog->getRHostField()->setFocus();
  setup_dialog->show();
}

qtchess_setup_dialog *qtchess_gui::getSetupDialog(void)
{
  return setup_dialog;
}

qtchess_history_dialog *qtchess_gui::getHistoryDialog(void)
{
  return history_dialog;
}

qtchess_promote_dialog *qtchess_gui::getPromoteDialog(void)
{
  return promote_dialog;
}

void qtchess_gui::notifyConnection(const QString &address)
{
  qapp->lock();
  gui->getStatusBar()->message("Status: Host " + address + " Connected");
  qapp->unlock();
}

void qtchess_gui::showGameOver(const int turn)
{
  (void) turn;
}

void qtchess_gui::showDisconnect(void)
{
  qapp->lock();
  setup_dialog->disconnectedState();
  gui->getStatusBar()->message("Status: Ready");
  qapp->unlock();
  clocks_dialog->stopTimers(PLAYER_TIMER);
  clocks_dialog->stopTimers(OPPONENT_TIMER);
}

void qtchess_gui::showNewGameInfo(void)
{
}

void qtchess_setup_dialog::ok_cb(void)
{
  QString str1 = "", str2 = "";

  /*
  ** Make sure the user is not connecting to this host.
  */

  str1 = rhost->text();
  str1 = str1.stripWhiteSpace();
  str2 = rport->text();
  str2 = str2.stripWhiteSpace();

  if(str1.length() == 0 || str2.length() == 0)
    return;

  comm->connectRemotely();

  if(comm->isSet())
    {
      ok->setEnabled(FALSE);
      rhost->setReadOnly(TRUE);
      rport->setReadOnly(TRUE);
      rhost->setEnabled(TRUE);
      rport->setEnabled(TRUE);
      disconnect->setEnabled(TRUE);
    }
  else
    {
      ok->setEnabled(TRUE);
      rhost->setReadOnly(FALSE);
      rport->setReadOnly(FALSE);
      rhost->setEnabled(TRUE);
      rport->setEnabled(TRUE);
      disconnect->setEnabled(FALSE);
    }
}

void qtchess_setup_dialog::close_cb(void)
{
  hide();
}

void qtchess_setup_dialog::disconnect_cb(void)
{
  if(comm->isSet())
    {
      comm->disconnectRemotely();

      if(!comm->isSet())
	{
	  gui->getStatusBar()->message("Status: Ready");
	  disconnectedState();
	}
    }
}

void qtchess_setup_dialog::disconnectedState(void)
{
  ok->setEnabled(TRUE);
  disconnect->setEnabled(FALSE);
  rhost->setReadOnly(FALSE);
  rport->setReadOnly(FALSE);
  rhost->setEnabled(TRUE);
  rport->setEnabled(TRUE);
}

qtchess_setup_dialog::qtchess_setup_dialog(QWidget *parent):
  QDialog(parent)
{
  QGroupBox *group_box = NULL;
  QGridLayout *layout1 = NULL;
  QHBoxLayout *layout2 = NULL;

  setCaption("Connection Configuration");

  if((layout1 = new QGridLayout(this, 2, 1, 0, -1, "layout1")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((layout2 = new QHBoxLayout(64)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((ok = new QPushButton("Connect", this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  ok->setMinimumWidth(105);

  if((disconnect = new QPushButton("Disconnect", this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  disconnect->setMinimumWidth(105);
  disconnect->setEnabled(FALSE);

  if((cancel = new QPushButton("Close", this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  cancel->setDefault(TRUE);
  cancel->setMinimumWidth(105);
  connect(cancel, SIGNAL(clicked()), this, SLOT(close_cb()));
  connect(ok, SIGNAL(clicked()), this, SLOT(ok_cb()));
  connect(disconnect, SIGNAL(clicked()), this, SLOT(disconnect_cb()));

  if((group_box = new QGroupBox(1, QGroupBox::Horizontal,
				"Information", this,
				"group_box")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((host_label = new QLabel("Local Host Name", group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((host = new QLineEdit(group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  host->setEnabled(TRUE);
  host->setReadOnly(TRUE);
  host->setMaxLength(256); // Not really necessary.

  if((port_label = new QLabel("Local Port Number", group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((port = new QLineEdit(group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((rhost_label = new QLabel("Remote Host Name", group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((rhost = new QLineEdit(group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((rport_label = new QLabel("Remote Port Number", group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((rport = new QLineEdit(group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  port->setEnabled(TRUE);
  port->setReadOnly(TRUE);
  rhost->setMaxLength(256);
  port->setMaxLength(16); // Not really necessary.
  rport->setMaxLength(16);
  layout2->addWidget(ok);
  layout2->addWidget(disconnect);
  layout2->addWidget(cancel);
  layout1->addWidget(group_box, 1, 1);
  layout1->addLayout(layout2, 2, 1);

  /*
  ** Prevent resizing.
  */

  layout()->setResizeMode(QLayout::Fixed);
}

QLineEdit *qtchess_setup_dialog::getHostField(void)
{
  return host;
}

QLineEdit *qtchess_setup_dialog::getPortField(void)
{
  return port;
}

QLineEdit *qtchess_setup_dialog::getRHostField(void)
{
  return rhost;
}

QLineEdit *qtchess_setup_dialog::getRPortField(void)
{
  return rport;
}

void qtchess_history_dialog::ok_cb(void)
{
  hide();
}

void qtchess_history_dialog::clear_cb(void)
{
  clear_method();
}

void qtchess_history_dialog::clear_method(void)
{
  text->clear();
}

void qtchess_history_dialog::add_move(const struct move_s current_move,
				      const int color)
{
  char msg[512];
  char txt[1024];
  char to_let[2];
  char from_let[2];

  (void) memset(msg, 0, sizeof(msg));
  (void) memset(txt, 0, sizeof(txt));
  (void) memset(to_let, 0, sizeof(to_let));
  (void) memset(from_let, 0, sizeof(from_let));
  from_let[0] = (char) (65 + current_move.y1);
  to_let[0] = (char ) (65 + current_move.y2);

#ifdef _DEBUG_
  (void) fprintf(stderr, "from_let = %s, to_let = %s\n", from_let, to_let);
#endif

  if(qtchess_validate::isKing(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d%s", "King",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1,
		    current_move.r_x1 != -1 ? " (castling)" : "");
  else if(qtchess_validate::isPawn(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Pawn",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);
  else if(qtchess_validate::isRook(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Rook",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);
  else if(qtchess_validate::isBishop(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Bishop",
		   from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);
  else if(qtchess_validate::isQueen(current_move.piece))
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Queen",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);
  else
    (void) snprintf(msg, sizeof(msg),
		    "%s from %s%d to %s%d", "Knight",
		    from_let, current_move.x1 + 1,
		    to_let, current_move.x2 + 1);

  if(color == BLACK)
    (void) snprintf(txt, sizeof(txt), "%d. Black %s", text->lines() + 1, msg);
  else
    (void) snprintf(txt, sizeof(txt), "%d. White %s", text->lines() + 1, msg);

  if(current_move.promoted)
    (void) strcat(txt, " (promotion)");

  (void) strcat(txt, ".");
  text->append(txt);
}

qtchess_history_dialog::qtchess_history_dialog(QWidget *parent):
  QDialog(parent)
{
  QGroupBox *group_box = NULL;
  QGridLayout *layout1 = NULL;
  QHBoxLayout *layout2 = NULL;

  setCaption("History");

  if((layout1 = new QGridLayout(this, 2, 1, 0, -1, "layout1")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((layout2 = new QHBoxLayout(64)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((ok = new QPushButton("Close", this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((clear = new QPushButton("Clear", this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  ok->setMinimumWidth(105);
  clear->setMaximumWidth(105);
  ok->setMaximumWidth(105);
  clear->setMinimumWidth(105);
  connect(ok, SIGNAL(clicked()), this, SLOT(ok_cb()));
  connect(clear, SIGNAL(clicked()), this, SLOT(clear_cb()));

  if((group_box = new QGroupBox(1, QGroupBox::Horizontal,
				"Moves", this,
				"group_box")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((text = new QTextEdit(group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  text->setTextFormat(QTextEdit::LogText);
  layout2->addWidget(clear);
  layout2->addWidget(ok);
  layout1->addWidget(group_box, 1, 1);
  layout1->addLayout(layout2, 2, 1);
  setGeometry(geometry().x(), geometry().y(), 325, 600);
}

void qtchess_promote_dialog::ok_cb(void)
{
  hide();
}

void qtchess_promote_dialog::setup(void)
{
  int h = 0, w = 0, x = 0, y = 0;

  h = gui->geometry().height();
  w = gui->geometry().width();
  x = gui->geometry().x();
  y = gui->geometry().y();
  hide();
  setGeometry(x + (w - geometry().width()) / 2,
	      y + (h - geometry().height()) / 2,
	      geometry().width(),
	      geometry().height());
  exec(); // Modal.
}

qtchess_promote_dialog::qtchess_promote_dialog(QWidget *parent):
  QDialog(parent)
{
  QLabel *label = NULL;
  QGroupBox *group_box = NULL;
  QGridLayout *layout1 = NULL;
  QHBoxLayout *layout2 = NULL;

  setCaption("Piece Promotion");

  if((layout1 = new QGridLayout(this, 2, 1, 0, -1, "layout1")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((layout2 = new QHBoxLayout(64)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((ok = new QPushButton("Promote", this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  ok->setMinimumWidth(85);
  ok->setMaximumWidth(85);
  connect(ok, SIGNAL(clicked()), this, SLOT(ok_cb()));

  if((group_box = new QGroupBox(1, QGroupBox::Horizontal,
				"Piece Selection", this,
				"group_box")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((label = new QLabel("Promotion Piece", group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((menu = new QComboBox(FALSE, group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  menu->insertItem("Rook");
  menu->insertItem("Queen");
  menu->insertItem("Bishop");
  menu->insertItem("Knight");
  layout2->addWidget(ok);
  layout1->addWidget(group_box, 1, 1);
  layout1->addLayout(layout2, 2, 1);

  /*
  ** Prevent resizing.
  */

  layout()->setResizeMode(QLayout::Fixed);
}

void qtchess_help_dialog::ok_cb(void)
{
  hide();
}

void qtchess_help_dialog::setup(void)
{
  int h = 0, w = 0, x = 0, y = 0;

  h = gui->geometry().height();
  w = gui->geometry().width();
  x = gui->geometry().x();
  y = gui->geometry().y();
  hide();
  setGeometry(x + (w - geometry().width()) / 2,
	      y + (h - geometry().height()) / 2,
	      geometry().width(),
	      geometry().height());
  exec(); // Modal.
}

qtchess_help_dialog::qtchess_help_dialog(QWidget *parent):
  QDialog(parent)
{
  QLabel *label = NULL;
  QGroupBox *group_box = NULL;
  QGridLayout *layout1 = NULL;
  QHBoxLayout *layout2 = NULL;

  setCaption("Help");

  if((layout1 = new QGridLayout(this, 2, 1, 0, -1, "layout1")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((layout2 = new QHBoxLayout(64)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((ok = new QPushButton("Close", this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  connect(ok, SIGNAL(clicked()), this, SLOT(ok_cb()));

  if((group_box = new QGroupBox(1, QGroupBox::Horizontal,
				"", this,
				"group_box")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((text = new QTextEdit(group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((label = new QLabel("", this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  text->setTextFormat(QTextEdit::LogText);
  layout2->addWidget(label);
  layout2->addWidget(label);
  layout2->addWidget(ok);
  layout2->addWidget(label);
  layout2->addWidget(label);
  layout1->addWidget(group_box, 1, 1);
  layout1->addLayout(layout2, 2, 1);
  setGeometry(geometry().x(), geometry().y(), 600, 300);
  text->append("General Guidelines\n"
	       "- Please do not connect to the same process.\n"
	       "- Beige always moves first.\n"
	       "- King is allowed to move to a threatened square, "
	       "unless castling.\n"
	       "- Moves are prohibited until connections have been "
	       "established.");
  layout()->setResizeMode(QLayout::Fixed);
}

void qtchess_clocks_dialog::ok_cb(void)
{
  hide();
}

void qtchess_gui::showClocks(void)
{
  clocks_dialog->show();
}

qtchess_clocks_dialog::qtchess_clocks_dialog(QWidget *parent):
  QDialog(parent)
{
  QLabel *label1 = NULL;
  QLabel *label2 = NULL;
  QGroupBox *group_box = NULL;
  QGridLayout *layout1 = NULL;
  QHBoxLayout *layout2 = NULL;
  QPushButton *ok = NULL;

  setCaption("Clocks");

  if((layout1 = new QGridLayout(this, 2, 1, 0, -1, "layout1")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((layout2 = new QHBoxLayout(64)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((ok = new QPushButton("Close", this)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  ok->setMinimumWidth(105);
  connect(ok, SIGNAL(clicked()), this, SLOT(ok_cb()));

  if((group_box = new QGroupBox(1, QGroupBox::Horizontal,
				"Clocks", this,
				"group_box")) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((label1 = new QLabel("Player", group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((player = new QTimeEdit(group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  player->setEnabled(TRUE);

  if((label2 = new QLabel("Opponent", group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  if((opponent = new QTimeEdit(group_box)) == NULL)
    chess->quit("Memory allocation failure.", EXIT_FAILURE);

  opponent->setEnabled(TRUE);
  layout2->addWidget(ok);
  layout1->addWidget(group_box, 1, 1);
  layout1->addLayout(layout2, 2, 1);

  /*
  ** Prevent resizing.
  */

  layout()->setResizeMode(QLayout::Fixed);
  connect(&playert, SIGNAL(timeout()), SLOT(updatePlayer()));
  connect(&opponentt, SIGNAL(timeout()), SLOT(updateOpponent()));
}

void qtchess_clocks_dialog::startTimers(const int which)
{
  if(which == PLAYER_TIMER)
    playert.start(1000);
  else
    opponentt.start(1000);
}

void qtchess_clocks_dialog::stopTimers(const int which)
{
  if(which == PLAYER_TIMER)
    playert.stop();
  else
    opponentt.stop();
}

void qtchess_clocks_dialog::updatePlayer(void)
{
  qapp->lock();
  player->setTime(player->time().addSecs(1));
  qapp->unlock();
}

void qtchess_clocks_dialog::updateOpponent(void)
{
  qapp->lock();
  opponent->setTime(opponent->time().addSecs(1));
  qapp->unlock();
}

void qtchess_gui::startTimers(const int which)
{
  clocks_dialog->startTimers(which);
}

void qtchess_gui::stopTimers(const int which)
{
  clocks_dialog->stopTimers(which);
}

void qtchess_clocks_dialog::init(void)
{
  playert.stop();
  opponentt.stop();
  player->setTime(QTime());
  opponent->setTime(QTime());
}

void qtchess_gui::initClocks(void)
{
  clocks_dialog->init();
}
